package com.ruoyi.common.core.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.PageDomain;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.page.TableSupport;
import com.ruoyi.common.exception.ASSERT;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.sql.SqlUtil;
import com.ruoyi.common.utils.translation.TranslateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;

import java.beans.PropertyEditorSupport;
import java.util.Date;
import java.util.List;

/**
 * web层通用数据处理
 *
 * @author ruoyi
 */
public class BaseController
{
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 将前台传递过来的日期格式的字符串，自动转化为Date类型
     */
    @InitBinder
    public void initBinder(WebDataBinder binder)
    {
        // Date 类型转换
        binder.registerCustomEditor(Date.class, new PropertyEditorSupport()
        {
            @Override
            public void setAsText(String text)
            {
                setValue(DateUtils.parseDate(text));
            }
        });
    }

    /**
     * 设置请求分页数据(允许不分页)
     */
    protected void startPage()
    {
        PageUtils.startPage(null, null);
    }

    /**
     * 设置请求分页数据(必须分页, 默认第1页取10条)
     */
    protected void mustPage()
    {
        PageUtils.startPage();
    }

    /**
     * 设置请求排序数据
     */
    protected void startOrderBy()
    {
        PageDomain pageDomain = TableSupport.buildPageRequest();
        if (StringUtils.isNotEmpty(pageDomain.getOrderBy()))
        {
            String orderBy = SqlUtil.escapeOrderBySql(pageDomain.getOrderBy());
            PageHelper.orderBy(orderBy);
        }
    }

    /**
     * 清理分页的线程变量
     */
    protected void clearPage()
    {
        PageUtils.clearPage();
    }

    /**
     * 响应请求分页数据
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
    protected TableDataInfo getDataTable(List<?> list)
    {
        translateList(list);
        TableDataInfo rspData = new TableDataInfo();
        rspData.setCode(HttpStatus.SUCCESS);
        rspData.setMsg("查询成功");
        rspData.setRows(list);
        rspData.setTotal(new PageInfo(list).getTotal());
        return rspData;
    }

    /**
     * 返回成功
     */
    public AjaxResult success()
    {
        return AjaxResult.success();
    }

    /**
     * 返回失败消息
     */
    public AjaxResult error()
    {
        return AjaxResult.error();
    }

    /**
     * 返回成功消息
     */
    public AjaxResult success(String message)
    {
        return AjaxResult.success(message);
    }

    /**
     * 返回成功消息
     */
    public AjaxResult success(Object data)
    {
        return AjaxResult.success(data);
    }

    /**
     * 返回失败消息
     */
    public AjaxResult error(String message)
    {
        return AjaxResult.error(message);
    }

    /**
     * 返回警告消息
     */
    public AjaxResult warn(String message)
    {
        return AjaxResult.warn(message);
    }

    /**
     * 响应返回结果
     *
     * @param rows 影响行数
     * @return 操作结果
     */
    protected AjaxResult toAjax(int rows)
    {
        return rows > 0 ? AjaxResult.success() : AjaxResult.error();
    }

    /**
     * 响应返回结果
     *
     * @param result 结果
     * @return 操作结果
     */
    protected AjaxResult toAjax(boolean result)
    {
        return result ? success() : error();
    }

    /**
     * 页面跳转
     */
    public String redirect(String url)
    {
        return StringUtils.format("redirect:{}", url);
    }

    /**
     * 获取用户缓存信息
     */
    public LoginUser getLoginUser()
    {
        return SecurityUtils.getLoginUser();
    }

    /**
     * 获取登录用户id
     */
    public Long getUserId()
    {
        return getLoginUser().getUserId();
    }
    /**
     * 获取登录用户昵称
     */
    public String getUserNickNmae()
    {
        return getLoginUser().getUser().getNickName();
    }

    /**
     * 获取登录用户手机号
     */
    public String getUserPhone()
    {
        return getLoginUser().getUser().getPhonenumber();
    }
    /**
     * 获取登录部门id
     */
    public Long getDeptId()
    {
        return getLoginUser().getDeptId();
    }

    /**
     * 获取登录用户名
     */
    public String getUsername()
    {
        return getLoginUser().getUsername();
    }

    protected void translateList(List<?> list) {
        boolean[] ret = {false};
        if (CollectionUtil.isNotEmpty(list) && ServletUtils.requestTranslateData(ret)) {
            try {
                TranslateUtils.translateList(list, ret[0]);
            } catch (Exception ignored) {
            }
        }
    }

    protected void MUST_AUTH()
    {
        ASSERT.EXP(getLoginUser().getUser().auth(), "用户未认证");
    }

    protected void MUST_FARMER()
    {
        ASSERT.EXP(getLoginUser().getUser().hasRole(SysRole.ROLE_KEY_FARMER), "农户才可操作");
    }

    protected void MUST_COMMITTEE()
    {
        ASSERT.EXP(getLoginUser().getUser().hasRole(SysRole.ROLE_KEY_VILLAGE_SOCRE), "村委才可操作");
    }

    protected void MUST_VILLAGE()
    {
        ASSERT.EXP(getLoginUser().getUser().hasRole(SysRole.ROLE_KEY_VILLAGE), "村管理员才可操作");
    }

    protected boolean IS_FARMER()
    {
        return Long.valueOf(SysRole.ROLE_ID_FARMER).equals(getUser().getCurrentRole());
    }

    protected boolean IS_COMMITTEE()
    {
        return Long.valueOf(SysRole.ROLE_ID_VILLAGE_SOCRE).equals(getUser().getCurrentRole());
    }

    protected boolean IS_VILLAGE()
    {
        return Long.valueOf(SysRole.ROLE_ID_VILLAGE).equals(getUser().getCurrentRole());
    }

    /**
     * 获取登录用户orgCode
     */
    public String getUserOrgCode()
    {
        return getLoginUser().getUser().getOrgCode();
    }

    public SysUser getUser()
    {
        return getLoginUser().getUser();
    }

    protected boolean userHasAboveTownRole()
    {
        SysUser user = getLoginUser().getUser();
        return user.hasRole(SysRole.ROLE_KEY_TOWN) || user.hasRole(SysRole.ROLE_KEY_COUNTY)  || user.hasRole(SysRole.ROLE_KEY_ADMIN);
    }
}
